@@ -1,3 +1,12 @@
+#
+#  Copyright (c) 2022 Russell Smiley
+#
+#  This file is part of pygitsync.
+#
+#  You should have received a copy of the MIT License along with pygitsync.
+#  If not, see <https://opensource.org/licenses/MIT>.
+#
+
 """Entry to the main execution path."""
 
 import asyncio
@@ -9,11 +18,13 @@ import typing
 
 import click
 import daemon  # type: ignore
+from click_logging_config import logging_parameters
 
 from ._configuration import DEFAULT_CONFIGURATION_FILE, _load_configuration
-from ._declarations import DEFAULT_LOG_LEVEL, VALID_LOG_LEVELS
-from ._logging import LoggingState
+from ._declarations import DEFAULT_LOG_CONFIG
 from ._repo import _fetch_repo
+from ._utility import Timer
+from ._version import __version__
 
 log = logging.getLogger(__name__)
 
@@ -73,23 +84,29 @@ async def _working_directory(
             yield working_directory
 
 
-async def main(
+async def enter_fetch_run_loop(
     configuration_file: pathlib.Path,
     is_daemon: bool,
     sleep_interval_seconds: typing.Optional[float],
     user_directory: typing.Optional[pathlib.Path],
+    post_fetch_task: typing.Optional[typing.Callable] = None,
 ) -> None:
-    """
-    Primary execution path of the utility.
+    """Primary execution path of the utility.
 
     ``is_daemon`` true means the loop will never exit, otherwise the loop only
     runs once and then exits.
 
+    The ``execute_this`` awaitable must not take any arguments and any return
+    values will be ignored.
+
     Args:
         configuration_file: Path to ``pygitsync`` configuration file.
         is_daemon: Flag to prevent execution from exiting.
         sleep_interval_seconds: Duration of period between git fetches.
         user_directory: User specified working directory. Created if necessary.
+        post_fetch_task: Awaitable function to execute after repo
+                         synchronization. Optional (default: does nothing and
+                         sleeps until the next repo sync).
     """
     async with _working_directory(user_directory) as working_directory:
         this_configuration = await _load_configuration(
@@ -98,11 +115,15 @@ async def main(
             sleep_interval_seconds,
         )
 
-        loop = WhileContext()
+        loop = WhileContext(not this_configuration.application.is_daemon)
         while loop.keep_running():
             try:
                 await _fetch_repo(this_configuration.repo, working_directory)
 
+                if post_fetch_task:
+                    with Timer("user defined process"):
+                        await post_fetch_task()
+
                 log.debug(
                     "sleeping for, "
                     f"{this_configuration.application.sleep_interval_seconds} "
@@ -149,27 +170,6 @@ async def main(
     show_default=True,
     type=float,
 )
-@click.option(
-    "--log-disable-console",
-    "disable_console_log",
-    default=False,
-    help="Disable console logging.",
-    is_flag=True,
-)
-@click.option(
-    "--log-enable-file",
-    "enable_file_log",
-    default=False,
-    help="Enable file logging.",
-    is_flag=True,
-)
-@click.option(
-    "--log-level",
-    "log_level",
-    default=DEFAULT_LOG_LEVEL,
-    help="Select logging level to apply to all enabled log sinks.",
-    type=click.Choice(VALID_LOG_LEVELS, case_sensitive=False),
-)
 @click.option(
     "--working",
     "-w",
@@ -180,21 +180,19 @@ async def main(
     show_default=False,
     type=click.Path(path_type=pathlib.Path),
 )
+@logging_parameters(DEFAULT_LOG_CONFIG)
 def process_cli_arguments(
     configuration_file: pathlib.Path,
-    disable_console_log: bool,
-    enable_file_log: bool,
     is_daemon: bool,
-    log_level: str,
     sleep_interval_seconds: typing.Optional[float],
     working_directory: typing.Optional[pathlib.Path],
 ) -> None:
     """Process CLI options."""
-    LoggingState((not enable_file_log), (not disable_console_log), log_level)
+    log.info(f"pygitsync version, {__version__}")
     if is_daemon:
         with daemon.DaemonContext():
             asyncio.run(
-                main(
+                enter_fetch_run_loop(
                     configuration_file,
                     is_daemon,
                     sleep_interval_seconds,
@@ -204,7 +202,7 @@ def process_cli_arguments(
     else:
         # run with the daemon context.
         asyncio.run(
-            main(
+            enter_fetch_run_loop(
                 configuration_file,
                 is_daemon,
                 sleep_interval_seconds,
